﻿Imports System
Imports System.Collections.Generic
Imports System.Text
Imports VoiceElements.Common
Imports VoiceElements.Client
Imports System.Threading
Imports System.IO

  Public Class IvrApplication
  Private Shared s_SyncVar As New Object()

  Public Shared ReadOnly Property SyncVar() As Object
    Get
      Return IvrApplication.s_SyncVar
    End Get
  End Property

  Private Shared s_Log As Log
  Public Shared ReadOnly Property Log() As Log
    Get
      Return s_Log
    End Get
  End Property

  Private Shared s_State As State

  Public Shared ReadOnly Property State() As State
    Get
      Return s_State
    End Get
  End Property

  Private Shared s_MainCodeThread As Thread

  Public Shared ReadOnly Property MainCodeThread() As Thread
    Get
      Return s_MainCodeThread
    End Get
  End Property

  Private Shared s_ThreadEvent As New AutoResetEvent(False)

  Public Shared ReadOnly Property ThreadEvent() As AutoResetEvent
    Get
      Return s_ThreadEvent
    End Get
  End Property

  Private Shared s_WorkingFolder As String = Nothing

  Public Shared ReadOnly Property WorkingFolder() As String
    Get
      Return s_WorkingFolder
    End Get
  End Property

  Shared Sub New()
    ' Constructor
    Dim currentDomain As AppDomain = AppDomain.CurrentDomain  'A&&&&
    s_Log = New Log("IvrApplication.Log")
    Log.Write("IvrApplication Constructor Complete")
    AddHandler currentDomain.UnhandledException, AddressOf CurrentDomain_UnhandledException
    'D&&&&AppDomain.CurrentDomain.UnhandledException += New UnhandledExceptionEventHandler(CurrentDomain_UnhandledException)
  End Sub

  Shared Sub CurrentDomain_UnhandledException(ByVal sender As Object, ByVal e As UnhandledExceptionEventArgs)
    If TypeOf e.ExceptionObject Is Exception Then
      Log.WriteException(DirectCast(e.ExceptionObject, Exception), "Domain Level Unhandled Exception")
    Else
      Log.Write("Domain Level Unhandled Exception - No Exception Object")
    End If
  End Sub

  Public Shared Sub Start()

    SyncLock SyncVar
      If State = State.Stopped Then
        s_State = State.Starting
        Dim ts As New ThreadStart(AddressOf MainCode)  'A&&&&
        'D&&&&Dim ts As New ThreadStart(MainCode)
        s_MainCodeThread = New Thread(ts)
        s_MainCodeThread.Name = "IvrApplication"
        s_MainCodeThread.Start()
        Log.Write("IvrApplication Starting...")
      Else
        Log.Write("IvrApplication is in the " + State.ToString() + " state.  Cannot start IvrApplication at this time.")
      End If
    End SyncLock
  End Sub

  Public Shared Sub StopImmediate()
    SyncLock SyncVar
      If State = State.Running OrElse State = State.StoppingControlled Then
        s_State = State.StoppingImmediate
        ThreadEvent.[Set]()
        Log.Write("IvrApplication StoppingImmediate.")
      Else
        Log.Write("IvrApplication is in the " + State.ToString() + " state.  Cannot stop IvrApplication at this time.")
      End If
    End SyncLock
  End Sub

  Public Shared Sub StopControlled()
    SyncLock SyncVar
      If State = State.Running Then
        s_State = State.StoppingControlled
        ThreadEvent.[Set]()
        Log.Write("IvrApplication StoppingControlled.")
      Else
        Log.Write("IvrApplication is in the " + State.ToString() + " state.  Cannot stop IvrApplication at this time.")
      End If
    End SyncLock
  End Sub

  Public Shared s_TelephonyServer As TelephonyServer = Nothing

  Public Shared Sub MainCode()

    Try
      s_WorkingFolder = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location)

      Log.Write("IvrApplication::MainCode() Starting...")

      ' Start Other Threads...
      Try

        ' UPDATE YOUR SERVER ADDRESS HERE
        Dim ips As System.Net.IPAddress() = System.Net.Dns.GetHostAddresses("bank.voiceelements.com")

        If ips Is Nothing OrElse ips.Length = 0 Then   'A&&&&
          'R&&&&If ips = Nothing OrElse ips.Length = 0 Then
          Throw New Exception("Error: Could not resolve Telephony Server specified!")
        End If

        Dim sIpaddress As String = "gtcp://" + ips(0).ToString() + ":54331"

        Log.Write("Connecting to: {0}", sIpaddress)

        ' CHANGE YOUR USERNAME AND PASSWORD HERE
                s_TelephonyServer = New TelephonyServer(sIpaddress, "username", "password")

        ' CHANGE YOUR CACHE MODE HERE
        '
        ' Client Session mode means that the server will stream and cache the files to/from your client machine.
        ' Files are flushed after you disconnect.
        '
        ' Server mode means that the files reside on the server and will use the full path name to find them there.
        ' Server mode can only be used on your own dedicate VE server.

        s_TelephonyServer.CacheMode = VoiceElements.[Interface].CacheMode.ClientSession
        's_TelephonyServer.CacheMode = VoiceElements.Interface.CacheMode.Server;

        ' SUBSCRIBE to the new call event.
        AddHandler s_TelephonyServer.NewCall, AddressOf s_TelephonyServer_NewCall  'A&&&&
        'D&&&&s_TelephonyServer.NewCall += New VoiceElements.Client.NewCall(s_TelephonyServer_NewCall)
        s_TelephonyServer.RegisterDNIS()

        ' Subscribe to the connection events to allow you to reconnect if something happens to the internet connection.
        ' If you are running your own VE server, this is less likely to happen except when you restart your VE server.
        AddHandler s_TelephonyServer.ConnectionLost, AddressOf s_TelephonyServer_ConnectionLost  'A&&&&
        'D&&&&s_TelephonyServer.ConnectionLost += New ConnectionLost(s_TelephonyServer_ConnectionLost)


        AddHandler s_TelephonyServer.ConnectionRestored, AddressOf s_TelephonyServer_ConnectionRestored  'A&&&&
        'D&&&&s_TelephonyServer.ConnectionRestored += New ConnectionRestored(s_TelephonyServer_ConnectionRestored)
      Catch ex As Exception
        Try
          'D&&&& If s_TelephonyServer <>s Nothing Then
          If Not (s_TelephonyServer Is Nothing) Then  'A&&&&
            s_TelephonyServer.Dispose()
          End If
        Catch generatedExceptionName As Exception
        End Try

        Log.Write("IvrApplication::MainCode() Exception: " + ex.Message + vbCr & vbLf + ex.StackTrace)
        Throw ex
      End Try

      Log.Write("VoiceElementsClient Version: {0}", s_TelephonyServer.GetClientVersion())
      Log.Write("VoiceElementsServer Version: {0}", s_TelephonyServer.GetServerVersion())

      SyncLock SyncVar
        s_State = State.Running
      End SyncLock

      Log.Write("IvrApplication::MainCode() Running...")

      While True

        ' Waits for some asyncronous event.
        ThreadEvent.WaitOne(10000, False)

        ' At this point you are in control.  You can farm out calls from a database, 
        ' or you could code the IvrInteractive Form and create a GUI for handling you calls.
        ' Follow the example from the Sampler on how to make an outbound class for new calls.

        SyncLock SyncVar
          If State <> State.Running Then
            Exit While
          End If
        End SyncLock
      End While

      s_TelephonyServer.Dispose()
      s_TelephonyServer = Nothing

      ' Must be shutting down...

      If State = State.StoppingControlled Then
        Log.Write("IvrApplication::MainCode() StoppingControlled...")
      End If

      If State = State.StoppingImmediate Then
        Log.Write("IvrApplication::MainCode() StoppingImmediate...")
      End If

      SyncLock SyncVar
        s_State = State.Stopped
        Log.Write("IvrApplication::MainCode() Stopped.")

      End SyncLock
    Catch ex As Exception
      Log.Write("IvrApplication::MainCode() Exception" + ex.Message + vbCr & vbLf + ex.StackTrace)
      s_State = State.Stopped
    Finally
      s_MainCodeThread = Nothing
    End Try
  End Sub

  Shared Sub s_TelephonyServer_ConnectionRestored(ByVal sender As Object, ByVal e As ConnectionRestoredEventArgs)

    ' When the connection is restored you must reset your cache mode and re-register the DNIS.

    s_TelephonyServer.CacheMode = VoiceElements.[Interface].CacheMode.ClientSession
    's_TelephonyServer.CacheMode = VoiceElements.Interface.CacheMode.Server;
    s_TelephonyServer.RegisterDNIS()

    Log.Write("The Connection to the server was successfully restored!")

  End Sub

  Shared Sub s_TelephonyServer_ConnectionLost(ByVal sender As Object, ByVal e As ConnectionLostEventArgs)

    ' You could also send an email to yourself to let you know that the server was down.
    Log.Write("The Connection to the server was lost.")
  End Sub

  Shared Sub ChannelResource_Disconnected(ByVal sender As Object, ByVal e As DisconnectedEventArgs)
    Log.Write("Disconnected (Meaning Caller Hung-up) Event Recevied.")
  End Sub

  Shared Sub s_TelephonyServer_NewCall(ByVal sender As Object, ByVal e As VoiceElements.Client.NewCallEventArgs)
    Try
      Log.Write("NewCall Arrival! DNIS: {0}  ANI: {1}  Caller ID Name: {2}", e.ChannelResource.Dnis, e.ChannelResource.Ani, e.ChannelResource.CallerIdName)

      ' Handle The New Call Here
      ' You should acutally create a class to handle this call.  See the Sampler application for how to do this.

      ' You can subscribe to get the disconnected event.
      AddHandler e.ChannelResource.Disconnected, AddressOf ChannelResource_Disconnected  'A&&&&
      'R&&&&e.ChannelResource.Disconnected += New Disconnected(ChannelResource_Disconnected)

      Log.Write("Answering...")

      e.ChannelResource.Answer()

      e.ChannelResource.VoiceResource.PlayTTS("Hello.  Welcome to Voice Elements.  Good bye.")
    Catch generatedExceptionName As HangupException
      Log.Write("The Caller Hungup.")
    Catch ex As Exception
      Log.WriteException(ex, "IvrApplication::NewCall")
    Finally
      ' Unsubscribe the event.
      RemoveHandler e.ChannelResource.Disconnected, AddressOf ChannelResource_Disconnected  'A&&&&
      'D&&&&e.ChannelResource.Disconnected -= New Disconnected(ChannelResource_Disconnected)
      ' Hangup
      e.ChannelResource.Disconnect()
      ' Always Dispose the object.
      e.ChannelResource.Dispose()


      Log.Write("Call complete.")
    End Try
  End Sub


End Class
